Param ( 
	[parameter(Mandatory=$true,Position=0)][string] $userGroup,
	[parameter(Mandatory=$true,Position=1)][string] $computerGroup,
	[parameter(Mandatory=$true,Position=2)][string] $domain)

Process {

	$account = $domain + "\" + $userGroup

	# get account information
	$getparams = @{Class="Win32_Account";Filter="Domain='$domain' and Name='$userGroup'"}
	$win32account = Get-WmiObject @getparams

	if ( $null -eq $win32account ) {
		throw "Account was not found: $account"
	}
	
	# get active directory hosts
	[System.Collections.ArrayList]$adGroupHosts = @()
	$adGroupMembers = get-adgroupmember -identity "$computerGroup" | select name, objectClass

	# filter hosts
	foreach ($adGroupMember in $adGroupMembers) {
		if ($adGroupMember.objectClass -eq "computer") {
			$adGroupHosts.Add($adGroupMember)
		}
	}
	Write-Host "found" $adGroupHosts.Count "hosts"

	if ($null -eq $adGroupHosts) {
		throw "No hosts found in domain $domain"
	}

	# configure hosts
	foreach ($adGroupHost in $adGroupHosts) {
		try {
			$args = @($account, $adGroupHost.name, $win32account.sid)
			Write-Host "Configure" $adGroupHost.name
			Invoke-Command -FilePath $PSScriptRoot\wmi.ps1 -ArgumentList $args -ComputerName $adGroupHost.name
		}
		catch {
			Write-Host "Failure for host" $adGroupHost.name
		}
	}
}
