Param ( [parameter(Mandatory=$true)][string] $account,
    [parameter(Mandatory=$true)][string] $computerName,
    [parameter(Mandatory=$true)][string] $groupSid,
    [System.Management.Automation.PSCredential] $credential = $null)
   
Process {
    $ErrorActionPreference = "Stop"
	
	$permissions = @("Enable", "RemoteAccess")
 
    # <DEF> set user access permissions
    Function Get-AccessMaskFromPermission($permissions) {
        $WBEM_ENABLE            = 1
                $WBEM_METHOD_EXECUTE = 2
                $WBEM_FULL_WRITE_REP   = 4
                $WBEM_PARTIAL_WRITE_REP              = 8
                $WBEM_WRITE_PROVIDER   = 0x10
                $WBEM_REMOTE_ACCESS    = 0x20
                $WBEM_RIGHT_SUBSCRIBE = 0x40
                $WBEM_RIGHT_PUBLISH      = 0x80
        $READ_CONTROL = 0x20000
        $WRITE_DAC = 0x40000
       
        $WBEM_RIGHTS_FLAGS = $WBEM_ENABLE,$WBEM_METHOD_EXECUTE,$WBEM_FULL_WRITE_REP,`
            $WBEM_PARTIAL_WRITE_REP,$WBEM_WRITE_PROVIDER,$WBEM_REMOTE_ACCESS,`
            $READ_CONTROL,$WRITE_DAC
        $WBEM_RIGHTS_STRINGS = "Enable","MethodExecute","FullWrite","PartialWrite",`
            "ProviderWrite","RemoteAccess","ReadSecurity","WriteSecurity"
 
        $permissionTable = @{}
 
        for ($i = 0; $i -lt $WBEM_RIGHTS_FLAGS.Length; $i++) {
            $permissionTable.Add($WBEM_RIGHTS_STRINGS[$i].ToLower(), $WBEM_RIGHTS_FLAGS[$i])
        }

        $accessMask = 0
 
        foreach ($permission in $permissions) {
            if (-not $permissionTable.ContainsKey($permission.ToLower())) {
                throw "Unknown permission: $permission`nValid permissions: $($permissionTable.Keys)"
            }
            $accessMask += $permissionTable[$permission.ToLower()]
        }
       
        $accessMask
    }

    
    # start
    ## <DEF> define how to invoke wmi on this computer
    if ($PSBoundParameters.ContainsKey("Credential")) {
        $remoteparams = @{ComputerName=$computer;Credential=$credential}
    } else {
        $remoteparams = @{ComputerName=$computerName}
    }
       
    $invokeparams = @{Namespace="root";Path="__systemsecurity=@"} + $remoteParams
 

    ## <READ> get security descriptor from wmi for this computer
    $output = Invoke-WmiMethod @invokeparams -Name GetSecurityDescriptor
    if ($output.ReturnValue -ne 0) {
        throw "GetSecurityDescriptor failed: $($output.ReturnValue)"
    }

    ## <READ> get computer name
    $computerName = (Get-WmiObject @remoteparams Win32_ComputerSystem).Name
   
    ## <DEF> define domain, accountname strings
    if ($account.Contains('\')) {
        $domainaccount = $account.Split('\')
        $domain = $domainaccount[0]
        if (($domain -eq ".") -or ($domain -eq "BUILTIN")) {
            $domain = $computerName
        }
        $accountname = $domainaccount[1]
    } elseif ($account.Contains('@')) {
        $domainaccount = $account.Split('@')
        $domain = $domainaccount[1].Split('.')[0]
        $accountname = $domainaccount[0]
    } else {
        $domain = $computerName
        $accountname = $account
    }
	
	## <DEF> define permission inheritance
    $acl = $output.Descriptor
    $OBJECT_INHERIT_ACE_FLAG = 0x1
    $CONTAINER_INHERIT_ACE_FLAG = 0x2
	
    ## <DEF> set user access permissions
	$accessMask = Get-AccessMaskFromPermission($permissions)

    ## <DEF> create access control object
	$ace = (New-Object System.Management.ManagementClass("win32_Ace")).CreateInstance()
	$ace.AccessMask = $accessMask
	
	### allow access control inheritance
	$ace.AceFlags = $CONTAINER_INHERIT_ACE_FLAG
	
    ## <DEF> access control object is for service user group (provided by ad.ps1 script)
	$trustee = (New-Object System.Management.ManagementClass("win32_Trustee")).CreateInstance()
	$trustee.SidString = $groupSid #$win32account.Sid
	$ace.Trustee = $trustee
   
    ## <DEF> allow access control enty (ACE) access
	$ACCESS_ALLOWED_ACE_TYPE = 0x0
    $ACCESS_DENIED_ACE_TYPE = 0x1 # unused
    
    $ace.AceType = $ACCESS_ALLOWED_ACE_TYPE

    $acl.DACL += $ace.psobject.immediateBaseObject
    
    ## <DEF> define wmi method parameters for access control
    $setparams = @{Name="SetSecurityDescriptor";ArgumentList=$acl.psobject.immediateBaseObject} + $invokeParams
 
    ## <WRITE> apply wmi method parameters for access control
    $output = Invoke-WmiMethod @setparams
    if ($output.ReturnValue -ne 0) {
        throw "SetSecurityDescriptor failed: $($output.ReturnValue)"
    }
    
    # prepare host
    $ram = Get-WmiObject Win32_PhysicalMemory
	$disks = Get-PhysicalDisk
}

# <DEF>     definitions
# <READ>    read from host
# <WRITE>   write to host